#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.equipment_port_input import EquipmentPortInput
from ..input.equipment_position_input import EquipmentPositionInput
from ..input.property_type_input import PropertyTypeInput


@dataclass(frozen=True)
class AddEquipmentTypeInput(DataClassJsonMixin):
    name: str
    positions: List[EquipmentPositionInput]
    ports: List[EquipmentPortInput]
    properties: List[PropertyTypeInput]
    category: Optional[str] = None
