#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class CheckListItemEnumSelectionMode(Enum):
    single = "single"
    multiple = "multiple"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "CheckListItemEnumSelectionMode":
        return cls.MISSING_ENUM
