#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.property_kind import PropertyKind

# fmt: off
QUERY: List[str] = ["""
fragment PropertyTypeFragment on PropertyType {
  id
  externalId
  name
  type
  index
  category
  rawValue
  isEditable
  isInstanceProperty
  isMandatory
  isDeleted
}

"""]

@dataclass(frozen=True)
class PropertyTypeFragment(DataClassJsonMixin):
    id: str
    externalId: Optional[str]
    name: str
    type: PropertyKind = _field(metadata=enum_field_metadata(PropertyKind))
    index: Optional[int]
    category: Optional[str]
    rawValue: Optional[str]
    isEditable: Optional[bool]
    isInstanceProperty: Optional[bool]
    isMandatory: Optional[bool]
    isDeleted: Optional[bool]
